"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
const functions_1 = require("./../utils/functions");
const lodash_1 = require("lodash");
const error_1 = __importDefault(require("./../utils/error"));
const functions_2 = require("./utils/functions");
const constants_1 = require("./../utils/constants");
module.exports = ({ strapi }) => ({
    getCommonService() {
        return (0, functions_1.getPluginService)("common");
    },
    async create(relation, data, user) {
        const { content, threadOf } = data;
        const singleRelationFulfilled = relation && constants_1.REGEX.relatedUid.test(relation);
        if (!singleRelationFulfilled) {
            throw new error_1.default(400, `Field "related" got incorrect format, use format like "api::<collection name>.<content type name>:<entity id>"`);
        }
        const [uid, relatedId] = await this.getCommonService().parseRelationString(relation);
        const enabledCollections = await this.getCommonService().getConfig(constants_1.CONFIG_PARAMS.ENABLED_COLLECTIONS, []);
        const isEnabledCollection = await this.getCommonService().isEnabledCollection(uid);
        if (enabledCollections.length > 0 && !isEnabledCollection) {
            throw new error_1.default(400, `Field "related" refer to not enabled collection. Please amend the plugin settings and try again`);
        }
        const relatedEntity = await strapi.db
            .query(uid)
            .findOne({ where: { id: relatedId } });
        if (!relatedEntity) {
            throw new error_1.default(400, `Relation for field "related" does not exist. Check your payload please.`);
        }
        const approvalFlow = await this.getCommonService().getConfig(constants_1.CONFIG_PARAMS.APPROVAL_FLOW, []);
        const isApprovalFlowEnabled = approvalFlow.includes(uid) || relatedEntity.requireCommentsApproval;
        const doNotPopulateAuthor = await this.getCommonService().getConfig(constants_1.CONFIG_PARAMS.AUTHOR_BLOCKED_PROPS, []);
        let linkToThread;
        try {
            linkToThread = !(0, lodash_1.isNil)(threadOf)
                ? await this.getCommonService().findOne({
                    id: threadOf,
                    related: relation,
                })
                : null;
        }
        catch (e) {
            throw new error_1.default(400, "Thread does not exist");
        }
        const validContext = this.getCommonService().isValidUserContext(user);
        if (!validContext) {
            throw (0, functions_2.resolveUserContextError)(user);
        }
        if ((await this.getCommonService().checkBadWords(content)) &&
            singleRelationFulfilled) {
            const { author, ...rest } = data;
            let authorData = {};
            if (validContext && user) {
                authorData = {
                    authorUser: user?.id,
                };
            }
            else if (author) {
                if (author.email && !constants_1.REGEX.email.test(author.email)) {
                    throw new error_1.default(400, 'Field: "author.email". Author e-mail is not valid value. Check your payload');
                }
                authorData = {
                    authorId: author.id,
                    authorName: author.name,
                    authorEmail: author.email,
                    authorAvatar: (0, lodash_1.isString)(author.avatar) ? author.avatar : undefined,
                };
            }
            const authorNotProperlyProvided = !(0, lodash_1.isEmpty)(authorData) && !(authorData.authorId || authorData.authorUser);
            if ((0, lodash_1.isEmpty)(authorData) || authorNotProperlyProvided) {
                throw new error_1.default(400, `Not able to recognise author of a comment. Make sure you've provided "author" property in a payload or authenticated your request properly.`);
            }
            if (isApprovalFlowEnabled &&
                !(0, lodash_1.isNil)(data.approvalStatus) &&
                data.approvalStatus !== constants_1.APPROVAL_STATUS.PENDING) {
                throw new error_1.default(400, "Invalid approval status");
            }
            const entity = await strapi.db
                .query((0, functions_2.getModelUid)("comment"))
                .create({
                data: {
                    ...rest,
                    ...authorData,
                    related: relation,
                    approvalStatus: isApprovalFlowEnabled
                        ? constants_1.APPROVAL_STATUS.PENDING
                        : null,
                },
                populate: {
                    authorUser: true,
                },
            });
            const sanitizedEntity = this.getCommonService().sanitizeCommentEntity({
                ...entity,
                threadOf: linkToThread,
            }, doNotPopulateAuthor);
            try {
                await this.sendResponseNotification(sanitizedEntity);
            }
            catch (e) {
                console.error(e);
            }
            return sanitizedEntity;
        }
        throw new error_1.default(400, "No content received");
    },
    async update(id, relation, data, user) {
        const { content } = data;
        const singleRelationFulfilled = relation && constants_1.REGEX.relatedUid.test(relation);
        if (!singleRelationFulfilled) {
            throw new error_1.default(400, `Request property "relation" got incorrect format, use format like "api::<collection name>.<content type name>:<entity id>"`);
        }
        await this.getCommonService().parseRelationString(relation);
        const existingEntity = await this.getCommonService().findOne({
            id,
            related: relation,
        });
        const validContext = this.getCommonService().isValidUserContext(user);
        if (!validContext) {
            throw (0, functions_2.resolveUserContextError)(user);
        }
        if ((0, functions_2.isEqualEntity)(existingEntity, data, user) && content) {
            if (await this.getCommonService().checkBadWords(content)) {
                const blockedAuthorProps = await this.getCommonService().getConfig(constants_1.CONFIG_PARAMS.AUTHOR_BLOCKED_PROPS, []);
                const entity = await strapi.db
                    .query((0, functions_2.getModelUid)("comment"))
                    .update({
                    where: { id },
                    data: { content },
                    populate: { threadOf: true, authorUser: true },
                });
                return this.getCommonService().sanitizeCommentEntity(entity, blockedAuthorProps);
            }
        }
        throw new error_1.default(403, `You're not allowed to take an action on that entity. Make sure you've provided "author" property in a payload or authenticated your request properly.`);
    },
    async reportAbuse(id, relation, payload, user) {
        if (!this.getCommonService().isValidUserContext(user)) {
            throw (0, functions_2.resolveUserContextError)(user);
        }
        await this.getCommonService().parseRelationString(relation);
        try {
            const reportAgainstEntity = await this.getCommonService().findOne({
                id,
                related: relation,
            });
            if (reportAgainstEntity.isAdminComment) {
                throw new error_1.default(403, `You're not allowed to take an action on that entity. This in a admin comment.`);
            }
            if (reportAgainstEntity) {
                const entity = await strapi.db
                    .query((0, functions_2.getModelUid)("comment-report"))
                    .create({
                    data: {
                        ...payload,
                        resolved: false,
                        related: id,
                    },
                });
                if (entity) {
                    const response = {
                        ...entity,
                        related: reportAgainstEntity,
                    };
                    try {
                        await this.sendAbuseReportEmail(entity.reason, entity.content);
                        return response;
                    }
                    catch (err) {
                        return response;
                    }
                }
                else {
                    throw new error_1.default(500, "Report cannot be created");
                }
            }
            throw new error_1.default(403, `You're not allowed to take an action on that entity. Make sure that comment exist or you've authenticated your request properly.`);
        }
        catch (e) {
            throw e;
        }
    },
    async markAsRemoved(id, relation, authorId, user) {
        if (!this.getCommonService().isValidUserContext(user)) {
            throw (0, functions_2.resolveUserContextError)(user);
        }
        const author = user?.id || authorId;
        if (!author) {
            throw new error_1.default(403, `You're not allowed to take an action on that entity. Make sure that you've provided proper "authorId" or authenticated your request properly.`);
        }
        await this.getCommonService().parseRelationString(relation);
        try {
            const byAuthor = user?.id
                ? {
                    authorUser: author,
                }
                : {
                    authorId: author,
                };
            const entity = await this.getCommonService().findOne({
                id,
                related: relation,
                ...byAuthor,
            });
            if (entity) {
                const removedEntity = await strapi.db
                    .query((0, functions_2.getModelUid)("comment"))
                    .update({
                    where: {
                        id,
                        related: relation,
                    },
                    data: { removed: true },
                    populate: { threadOf: true, authorUser: true },
                });
                await this.markAsRemovedNested(id, true);
                const doNotPopulateAuthor = await this.getCommonService().getConfig(constants_1.CONFIG_PARAMS.AUTHOR_BLOCKED_PROPS, []);
                return this.getCommonService().sanitizeCommentEntity(removedEntity, doNotPopulateAuthor);
            }
            else {
                throw new error_1.default(404, `Entity does not exist or you're not allowed to take an action on it`);
            }
        }
        catch (e) {
            throw new error_1.default(404, `Entity does not exist or you're not allowed to take an action on it`);
        }
    },
    async sendAbuseReportEmail(reason, content) {
        const rolesToBeNotified = (await this.getCommonService().getConfig(constants_1.CONFIG_PARAMS.MODERATOR_ROLES)) || ["strapi-super-admin"];
        const adminUserModel = strapi.db.query("admin::user");
        const emails = await adminUserModel
            .findMany({
            where: {
                roles: { code: rolesToBeNotified },
            },
        })
            .then((items) => items.map((_) => _.email));
        const superAdmin = await strapi.db
            .query("admin::user")
            .findOne({
            where: {
                roles: { code: "strapi-super-admin" },
            },
        });
        if (emails.length > 0) {
            try {
                await strapi
                    .plugin("email")
                    .service("email")
                    .send({
                    to: emails,
                    from: superAdmin.email,
                    subject: "New abuse report on comment",
                    text: `
                        There was a new abuse report on your app. 
                        Reason: ${reason}
                        Message: ${content}
                    `,
                });
            }
            catch (err) {
                strapi.log.error(err);
                throw err;
            }
        }
    },
    async markAsRemovedNested(id, status) {
        return this.getCommonService().modifiedNestedNestedComments(id, "removed", status);
    },
    async sendResponseNotification(entity) {
        if (entity.threadOf) {
            const thread = (0, lodash_1.isObject)(entity.threadOf) ? entity.threadOf : await this.getCommonService().findOne({ id: entity.threadOf });
            let emailRecipient = thread?.author?.email;
            if (thread.authorUser && !emailRecipient) {
                const strapiUser = (0, lodash_1.isObject)(thread.authorUser) ? thread.authorUser : await strapi.db.query("api::user").findOne({
                    where: { id: thread.authorUser }
                });
                emailRecipient = strapiUser?.email;
            }
            if (emailRecipient) {
                const superAdmin = await strapi.db
                    .query("admin::user")
                    .findOne({
                    where: {
                        roles: { code: "strapi-super-admin" },
                    },
                });
                const emailSender = await this.getCommonService().getConfig('client.contactEmail', superAdmin.email);
                const clientAppUrl = await this.getCommonService().getConfig('client.url', 'our site');
                try {
                    await strapi
                        .plugin("email")
                        .service("email")
                        .send({
                        to: [emailRecipient],
                        from: emailSender,
                        subject: "You've got a new response to your comment",
                        text: `Hello ${thread?.author?.name || emailRecipient}!
                You've got a new response to your comment by ${entity?.author?.name || entity?.author?.email}.
                
                ------

                "${entity.content}"

                ------
                
                Visit ${clientAppUrl} and continue the discussion.
                `,
                    });
                }
                catch (err) {
                    strapi.log.error(err);
                    throw err;
                }
            }
        }
    },
});
//# sourceMappingURL=client.js.map