"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
const bad_words_1 = __importDefault(require("bad-words"));
const lodash_1 = require("lodash");
const constants_1 = require("../utils/constants");
const error_1 = __importDefault(require("./../utils/error"));
const functions_1 = require("./utils/functions");
const parsers_1 = require("./utils/parsers");
const PAGE_SIZE = 10;
module.exports = ({ strapi }) => ({
    async getConfig(prop, defaultValue, useLocal = false) {
        const queryProp = (0, functions_1.buildConfigQueryProp)(prop);
        const pluginStore = await this.getPluginStore();
        const config = await pluginStore.get({
            key: "config",
        });
        let result;
        if (config && !useLocal) {
            result = queryProp ? (0, lodash_1.get)(config, queryProp, defaultValue) : config;
        }
        else {
            result = this.getLocalConfig(queryProp, defaultValue);
        }
        return (0, lodash_1.isNil)(result) ? defaultValue : result;
    },
    async getPluginStore() {
        return strapi.store({ type: "plugin", name: "comments" });
    },
    getLocalConfig(prop, defaultValue) {
        const queryProp = (0, functions_1.buildConfigQueryProp)(prop);
        const result = strapi.config.get(`plugin.comments${queryProp ? "." + queryProp : ""}`);
        return (0, lodash_1.isNil)(result) ? defaultValue : result;
    },
    async findAllFlat({ query = {}, populate = {}, sort, pagination, fields, isAdmin = false, }, relatedEntity = null) {
        const defaultSelect = ["id", "related"];
        const populateClause = {
            authorUser: true,
            ...((0, lodash_1.isObject)(populate) ? populate : {}),
        };
        const doNotPopulateAuthor = isAdmin
            ? []
            : await this.getConfig(constants_1.CONFIG_PARAMS.AUTHOR_BLOCKED_PROPS, []);
        const sortQuery = (0, parsers_1.parseSortQuery)(sort);
        const fieldsQuery = (0, parsers_1.parseFieldsQuery)(fields, sortQuery, defaultSelect);
        let [meta, queryExtension = {}] = (0, parsers_1.parsePaginationsQuery)(pagination, fieldsQuery, { PAGE_SIZE });
        const entries = await strapi.db
            .query((0, functions_1.getModelUid)("comment"))
            .findMany({
            where: {
                ...query,
            },
            populate: {
                ...populateClause,
            },
            ...queryExtension,
        });
        if (pagination?.withCount &&
            (pagination.withCount === "true" || pagination.withCount === true)) {
            const total = await strapi.db
                .query((0, functions_1.getModelUid)("comment"))
                .count({
                where: {
                    ...query,
                },
            });
            const pageSize = meta.pagination.pageSize || PAGE_SIZE;
            const pageCount = Math.floor(total / pageSize);
            meta = {
                ...meta,
                pagination: {
                    ...meta.pagination,
                    pageCount: !(0, lodash_1.isNil)(meta.pagination.page)
                        ? total % pageSize === 0
                            ? pageCount
                            : pageCount + 1
                        : undefined,
                    total,
                },
            };
        }
        const entriesWithThreads = await Promise.all(entries.map(async (_) => {
            const [nestedEntries, count] = await strapi.db
                .query((0, functions_1.getModelUid)("comment"))
                .findWithCount({
                where: {
                    threadOf: _.id,
                },
            });
            return {
                id: _.id,
                itemsInTread: count,
                firstThreadItemId: (0, lodash_1.first)(nestedEntries)?.id,
            };
        }));
        const relatedEntities = relatedEntity !== null
            ? [relatedEntity]
            : await this.findRelatedEntitiesFor([...entries]);
        const hasRelatedEntitiesToMap = relatedEntities.filter((_) => _).length > 0;
        const result = entries.map((_) => {
            const threadedItem = entriesWithThreads.find((item) => item.id === _.id);
            const parsedThreadOf = (0, lodash_1.isString)(query.threadOf)
                ? (0, lodash_1.parseInt)(query.threadOf)
                : query.threadOf;
            let authorUserPopulate = {};
            if ((0, lodash_1.isObject)(populateClause?.authorUser)) {
                authorUserPopulate =
                    "populate" in populateClause.authorUser
                        ? populateClause.authorUser.populate
                        : populateClause.authorUser;
            }
            const primitiveThreadOf = (0, lodash_1.isString)(parsedThreadOf) || (0, lodash_1.isNumber)(parsedThreadOf) ?
                parsedThreadOf :
                null;
            return this.sanitizeCommentEntity({
                ..._,
                threadOf: primitiveThreadOf || _.threadOf,
                gotThread: (threadedItem?.itemsInTread || 0) > 0,
                threadFirstItemId: threadedItem?.firstThreadItemId,
            }, doNotPopulateAuthor, authorUserPopulate);
        });
        return {
            data: hasRelatedEntitiesToMap
                ? result.map((_) => this.mergeRelatedEntityTo(_, relatedEntities))
                : result,
            ...((0, lodash_1.isEmpty)(meta) ? {} : { meta }),
        };
    },
    async findAllInHierarchy({ query, populate = {}, sort, fields, startingFromId = null, dropBlockedThreads = false, isAdmin = false, }, relatedEntity) {
        const entities = await this.findAllFlat({ query, populate, sort, fields, isAdmin }, relatedEntity);
        return (0, functions_1.buildNestedStructure)(entities?.data, startingFromId, "threadOf", dropBlockedThreads, false);
    },
    async findOne(criteria) {
        const entity = await strapi.db
            .query((0, functions_1.getModelUid)("comment"))
            .findOne({
            where: criteria,
            populate: {
                reports: true,
                authorUser: true,
            },
        });
        if (!entity) {
            throw new error_1.default(404, "Comment does not exist. Check your payload please.");
        }
        const doNotPopulateAuthor = await this.getConfig(constants_1.CONFIG_PARAMS.AUTHOR_BLOCKED_PROPS, []);
        return (0, functions_1.filterOurResolvedReports)(this.sanitizeCommentEntity(entity, doNotPopulateAuthor));
    },
    async findAllPerAuthor({ query = {}, populate = {}, pagination, sort, fields, isAdmin = false, }, authorId, isStrapiAuthor = false) {
        if ((0, lodash_1.isNil)(authorId)) {
            return {
                data: [],
            };
        }
        const { related, ...restQuery } = query;
        const authorQuery = isStrapiAuthor ? {
            authorUser: {
                id: authorId
            },
        } : {
            authorId,
        };
        const response = await this.findAllFlat({
            query: {
                ...restQuery,
                ...authorQuery,
            },
            pagination, populate, sort, fields, isAdmin
        });
        return {
            ...response,
            data: response.data.map(({ author, ...rest }) => rest),
        };
    },
    async findRelatedEntitiesFor(entities = []) {
        const data = entities.reduce((acc, curr) => {
            const [relatedUid, relatedStringId] = (0, functions_1.getRelatedGroups)(curr.related);
            const parsedRelatedId = (0, lodash_1.parseInt)(relatedStringId);
            const relatedId = (0, lodash_1.isNumber)(parsedRelatedId)
                ? parsedRelatedId
                : relatedStringId;
            return {
                ...acc,
                [relatedUid]: [...(acc[relatedUid] || []), relatedId],
            };
        }, {});
        return Promise.all(Object.entries(data).map(async ([relatedUid, relatedStringIds]) => strapi.db
            .query(relatedUid)
            .findMany({
            where: { id: Array.from(new Set(relatedStringIds)) },
        })
            .then((relatedEntities) => relatedEntities.map((_) => ({
            ..._,
            uid: relatedUid,
        }))))).then((result) => result.flat(2));
    },
    mergeRelatedEntityTo(entity, relatedEntities = []) {
        return {
            ...entity,
            related: relatedEntities.find((relatedEntity) => entity.related === `${relatedEntity.uid}:${relatedEntity.id}`),
        };
    },
    async modifiedNestedNestedComments(id, fieldName, value) {
        try {
            const entitiesToChange = await strapi.db
                .query((0, functions_1.getModelUid)("comment"))
                .findMany({
                where: { threadOf: id },
            });
            const changedEntities = await strapi.db
                .query((0, functions_1.getModelUid)("comment"))
                .updateMany({
                where: { id: entitiesToChange.map((_) => _.id) },
                data: { [fieldName]: value },
            });
            if (entitiesToChange.length === changedEntities.count &&
                changedEntities.count > 0) {
                const nestedTransactions = await Promise.all(entitiesToChange.map((item) => this.modifiedNestedNestedComments(item.id, fieldName, value)));
                return nestedTransactions.length === changedEntities.count;
            }
            return true;
        }
        catch (e) {
            return false;
        }
    },
    sanitizeCommentEntity(entity, blockedAuthorProps, populate) {
        const fieldsToPopulate = (0, lodash_1.isArray)(populate)
            ? populate
            : Object.keys(populate || {});
        return {
            ...(0, functions_1.buildAuthorModel)({
                ...entity,
                threadOf: (0, lodash_1.isObject)(entity.threadOf)
                    ? (0, functions_1.buildAuthorModel)(entity.threadOf, blockedAuthorProps, fieldsToPopulate)
                    : entity.threadOf,
            }, blockedAuthorProps, fieldsToPopulate),
        };
    },
    isValidUserContext(user) {
        return user ? !(0, lodash_1.isNil)(user?.id) : true;
    },
    async parseRelationString(relation) {
        const [uid, relatedStringId] = (0, functions_1.getRelatedGroups)(relation);
        const parsedRelatedId = (0, lodash_1.parseInt)(relatedStringId);
        const relatedId = (0, lodash_1.isNumber)(parsedRelatedId)
            ? parsedRelatedId
            : relatedStringId;
        const isEnabledCollection = await this.isEnabledCollection(uid);
        const enabledCollections = await this.getConfig(constants_1.CONFIG_PARAMS.ENABLED_COLLECTIONS, []);
        if (enabledCollections.length > 0 && !isEnabledCollection) {
            throw new error_1.default(403, `Action not allowed for collection '${uid}'. Use one of: ${enabledCollections.join(", ")}`);
        }
        return [uid, relatedId];
    },
    async checkBadWords(content) {
        const config = await this.getConfig(constants_1.CONFIG_PARAMS.BAD_WORDS, true);
        if (config) {
            const filter = new bad_words_1.default((0, lodash_1.isObject)(config) ? config : undefined);
            if (content && filter.isProfane(content)) {
                throw new error_1.default(400, "Bad language used! Please polite your comment...", {
                    content: {
                        original: content,
                        filtered: content && filter.clean(content),
                    },
                });
            }
        }
        return content;
    },
    async isEnabledCollection(uid) {
        const enabledCollections = await this.getConfig("enabledCollections", []);
        return enabledCollections.includes(uid);
    },
});
//# sourceMappingURL=common.js.map