"use strict";
const fp_1 = require("lodash/fp");
const nexus_1 = require("nexus");
const functions_1 = require("../utils/functions");
const virtualScalarAttributes = ["id"];
const customFields = ["filterBy", "filterByValue"];
module.exports = ({ strapi }) => {
    const { service: getService } = strapi.plugin("graphql");
    const rootLevelOperators = () => {
        const { operators } = strapi.plugin("graphql").service("builders").filters;
        return [operators.and, operators.or, operators.not];
    };
    const buildContentTypeFilters = (contentType) => {
        const utils = strapi.plugin("graphql").service("utils");
        const extension = strapi.plugin("graphql").service("extension");
        const { getFiltersInputTypeName, getScalarFilterInputTypeName } = utils.naming;
        const { isStrapiScalar, isRelation } = utils.attributes;
        const { attributes } = contentType;
        const filtersTypeName = getFiltersInputTypeName(contentType);
        return (0, nexus_1.inputObjectType)({
            name: filtersTypeName,
            definition(t) {
                const validAttributes = Object.entries(attributes).filter(([attributeName]) => extension
                    .shadowCRUD(contentType.uid)
                    .field(attributeName)
                    .hasFiltersEnabeld());
                const isIDFilterEnabled = extension
                    .shadowCRUD(contentType.uid)
                    .field("id")
                    .hasFiltersEnabeld();
                if (contentType.kind === "collectionType" && isIDFilterEnabled) {
                    t.field("id", { type: getScalarFilterInputTypeName("ID") });
                }
                for (const [attributeName, attribute] of validAttributes) {
                    if (isStrapiScalar(attribute)) {
                        addScalarAttribute(t, attributeName, attribute);
                    }
                    else if (isRelation(attribute)) {
                        addRelationalAttribute(t, attributeName, attribute);
                    }
                }
                for (const operator of rootLevelOperators()) {
                    operator.add(t, filtersTypeName);
                }
                t.field("filterBy", { type: "String" });
                t.field("filterByValue", { type: "String" });
            },
        });
    };
    const addScalarAttribute = (builder, attributeName, attribute) => {
        const { naming, mappers } = strapi.plugin("graphql").service("utils");
        const gqlType = mappers.strapiScalarToGraphQLScalar(attribute.type);
        builder.field(attributeName, {
            type: naming.getScalarFilterInputTypeName(gqlType),
        });
    };
    const addRelationalAttribute = (builder, attributeName, attribute) => {
        const utils = strapi.plugin("graphql").service("utils");
        const extension = strapi.plugin("graphql").service("extension");
        const { getFiltersInputTypeName } = utils.naming;
        const { isMorphRelation } = utils.attributes;
        const model = strapi.getModel(attribute.target);
        if (!model || isMorphRelation(attribute))
            return;
        if (extension.shadowCRUD(model.uid).isDisabled())
            return;
        builder.field(attributeName, { type: getFiltersInputTypeName(model) });
    };
    const recursivelyReplaceScalarOperators = (data) => {
        const { operators } = getService("builders").filters;
        if (Array.isArray(data)) {
            return data.map(recursivelyReplaceScalarOperators);
        }
        if ((0, fp_1.isDate)(data) || !(0, fp_1.isObject)(data)) {
            return data;
        }
        const result = {};
        for (const [key, value] of Object.entries(data)) {
            const isOperator = !!operators[key];
            const newKey = isOperator ? operators[key].strapiOperator : key;
            result[newKey] = recursivelyReplaceScalarOperators(value);
        }
        return result;
    };
    const graphQLFiltersToStrapiQuery = (queryFilters, contentType = {}) => {
        const { isStrapiScalar, isMedia, isRelation } = getService("utils").attributes;
        const { operators } = getService("builders").filters;
        const ROOT_LEVEL_OPERATORS = [operators.and, operators.or, operators.not];
        if ((0, fp_1.isNil)(queryFilters)) {
            return {};
        }
        if (Array.isArray(queryFilters)) {
            return queryFilters.reduce((acc, filtersItem) => {
                if (!customFields.includes(filtersItem)) {
                    acc.push(graphQLFiltersToStrapiQuery(filtersItem, contentType));
                }
                return acc;
            });
        }
        const resultMap = {};
        const { filterBy, filterByValue, ...filters } = queryFilters;
        const { attributes } = contentType;
        const isAttribute = (attributeName) => {
            return (virtualScalarAttributes.includes(attributeName) ||
                (0, fp_1.has)(attributeName, attributes));
        };
        for (const [key, value] of Object.entries(filters)) {
            if (isAttribute(key)) {
                const attribute = attributes[key];
                if (virtualScalarAttributes.includes(key) ||
                    isStrapiScalar(attribute)) {
                    resultMap[key] = recursivelyReplaceScalarOperators(value);
                }
                else if (isRelation(attribute) || isMedia(attribute)) {
                    const relModel = strapi.getModel(attribute.target);
                    resultMap[key] = graphQLFiltersToStrapiQuery(value, relModel);
                }
            }
            else {
                const rootLevelOperator = ROOT_LEVEL_OPERATORS.find((0, fp_1.propEq)("fieldName", key));
                if (rootLevelOperator) {
                    const { strapiOperator } = rootLevelOperator;
                    resultMap[strapiOperator] = graphQLFiltersToStrapiQuery(value, contentType);
                }
            }
        }
        if (filterBy === "DATE_CREATED") {
            const date = new Date(filterByValue);
            if (!filterByValue || Number.isNaN(+date)) {
                throw new Error('Invalid date specified in "filterByValue"');
            }
            const start = date.setHours(0, 0, 0, 0);
            const end = date.setHours(23, 59, 59, 999);
            resultMap.createdAt = {
                $between: [start, end],
            };
        }
        if (filterBy === "APPROVAL_STATUS") {
            (0, functions_1.assertNotEmpty)(filterByValue, new Error('Empty "filterByValue" parameter'));
            resultMap.approvalStatus = filterByValue;
        }
        return resultMap;
    };
    const serviceMethods = {
        buildContentTypeFilters,
        graphQLFiltersToStrapiQuery,
    };
    return serviceMethods;
};
//# sourceMappingURL=graphql.js.map